@@ -1,6 +1,6 @@
 # -*- encoding: utf8 -*-
 #
-# $Id: html.py 4322 2010-03-26 03:57:18Z rjones $
+# $Id: html.py 4814 2010-09-07 03:52:24Z rjones $
 # $HeadURL: svn+ssh://svn/svn/trunk/api/eklib/html.py $
 #
 '''Simple, elegant HTML generation.
@@ -158,6 +158,8 @@ with the appropriate XHTML minimized tag syntax. For example:
 Version History (in Brief)
 --------------------------
 
+- 1.12 fix handling of XHTML empty tags when generating unicode
+  output (thanks Carsten Eggers)
 - 1.11 remove setuptools dependency
 - 1.10 support plain ol' distutils again
 - 1.9 added unicode support for Python 2.x
@@ -173,22 +175,23 @@ Version History (in Brief)
 
 ----
 
-This code is copyright 2009 eKit.com Inc (http://www.ekit.com/)
+This code is copyright 2009-2010 eKit.com Inc (http://www.ekit.com/)
 See the end of the source file for the license of use.
 XHTML support was contributed by Michael Haubenwallner.
 '''
 from __future__ import with_statement
-__version__ = '1.11'
+__version__ = '1.12'
 
 import unittest
 import cgi
 
+
 class HTML(object):
     '''Easily generate HTML.
 
     '''
     newline_default_on = set('table ol ul dl'.split())
-        
+
     def __init__(self, name=None, stack=None, newlines=True):
         self.name = name
         self.content = []
@@ -202,6 +205,7 @@ class HTML(object):
             self.top = False
             self.newlines = name in self.newline_default_on
         self.stack = stack
+
     def __getattr__(self, name):
         # adding a new tag or newline
         if name == 'newline':
@@ -213,6 +217,7 @@ class HTML(object):
         else:
             self.content.append(e)
         return e
+
     def text(self, text, escape=True):
         '''Add text to the document. If "escape" is True any characters
         special to HTML will be escaped.
@@ -224,11 +229,13 @@ class HTML(object):
             self.stack[-1].content.append(text)
         else:
             self.content.append(text)
+
     def raw_text(self, text):
         '''Add raw, unescaped text to the document. This is useful for
         explicitly adding HTML code or entities.
         '''
         return self.text(text, escape=False)
+
     def __call__(self, *content, **kw):
         # customising a tag with content or attributes
         escape = kw.pop('escape', True)
@@ -246,42 +253,50 @@ class HTML(object):
             else:
                 self.attrs[k] = cgi.escape(kw[k], True)
         return self
+
     def __enter__(self):
         # we're now adding tags to me!
         self.stack.append(self)
         return self
+
     def __exit__(self, exc_type, exc_value, exc_tb):
         # we're done adding tags to me!
         self.stack.pop()
+
     def __repr__(self):
-        return '<HTML %s 0x%x>'%(self.name, id(self))
+        return '<HTML %s 0x%x>' % (self.name, id(self))
+
     def stringify(self, str_type):
         # turn me and my content into text
         join = '\n' if self.newlines else ''
         if self.name is None:
             return join.join(map(str_type, self.content))
-        a = ['%s="%s"'%i for i in self.attrs.items()]
+        a = ['%s="%s"' % i for i in self.attrs.items()]
         l = [self.name] + a
-        s = '<%s>%s'%(' '.join(l), join)
+        s = '<%s>%s' % (' '.join(l), join)
         if self.content:
-            s += join.join(map(str_type, self.content))   
-            s += join + '</%s>'%self.name
+            s += join.join(map(str_type, self.content))
+            s += join + '</%s>' % self.name
         return s
+
     def __str__(self):
         return self.stringify(str)
+
     def __unicode__(self):
         return self.stringify(unicode)
+
     def __iter__(self):
         return iter([str(self)])
 
+
 class XHTML(HTML):
     '''Easily generate XHTML.
 
     '''
-    
+
     empty_elements = set('base meta link hr br param img area input col \
         colgroup basefont isindex frame'.split())
-    
+
     def __getattr__(self, name):
         # adding a new tag or newline
         if name == 'newline':
@@ -293,39 +308,49 @@ class XHTML(HTML):
         else:
             self.content.append(e)
         return e
-    def __str__(self):
+
+    def stringify(self, str_type):
         # turn me and my content into text
         # honor empty and non-empty elements
         join = '\n' if self.newlines else ''
         if self.name is None:
-            return join.join(map(str, self.content))
-        a = ['%s="%s"'%i for i in self.attrs.items()]
+            return join.join(map(str_type, self.content))
+        a = ['%s="%s"' % i for i in self.attrs.items()]
         l = [self.name] + a
-        s = '<%s>%s'%(' '.join(l), join)
+        s = '<%s>%s' % (' '.join(l), join)
         if self.content or not(self.name.lower() in self.empty_elements):
-            s += join.join(map(str, self.content))   
-            s += join + '</%s>'%self.name
+            s += join.join(map(str_type, self.content))
+            s += join + '</%s>' % self.name
         else:
-            s = '<%s />%s'%(' '.join(l), join)
+            s = '<%s />%s' % (' '.join(l), join)
         return s
 
+
 class TestCase(unittest.TestCase):
     def test_empty_tag(self):
         'generation of an empty HTML tag'
-        h = HTML()
-        h.br
-        self.assertEquals(str(h), '<br>')
+        self.assertEquals(str(HTML().br), '<br>')
 
-    def test_empty_tag(self):
+    def test_empty_tag_unicode(self):
+        'generation of an empty HTML tag'
+        self.assertEquals(unicode(HTML().br), u'<br>')
+
+    def test_empty_tag_xml(self):
         'generation of an empty XHTML tag'
-        h = XHTML()
-        h.br
-        self.assertEquals(str(h), '<br />')
- 
+        self.assertEquals(str(XHTML().br), '<br />')
+
+    def test_empty_tag_xml_unicode(self):
+        'generation of an empty XHTML tag'
+        self.assertEquals(unicode(XHTML().br), u'<br />')
+
     def test_xhtml_match_tag(self):
         'check forced generation of matching tag when empty'
         self.assertEquals(str(XHTML().p), '<p></p>')
- 
+
+    def test_xhtml_match_tag_unicode(self):
+        'check forced generation of matching tag when empty'
+        self.assertEquals(unicode(XHTML().p), u'<p></p>')
+
     def test_just_tag(self):
         'generate HTML for just one tag'
         self.assertEquals(str(HTML().br), '<br>')
@@ -372,7 +397,8 @@ class TestCase(unittest.TestCase):
         l = h.ol
         l.li('foo')
         l.li.b('bar')
-        self.assertEquals(str(h), '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
+        self.assertEquals(str(h),
+            '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
 
     def test_subtag_direct_context(self):
         'generation of sub-tags directly on the parent tag in "with" context'
@@ -380,7 +406,8 @@ class TestCase(unittest.TestCase):
         with h.ol as l:
             l.li('foo')
             l.li.b('bar')
-        self.assertEquals(str(h), '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
+        self.assertEquals(str(h),
+            '<ol>\n<li>foo</li>\n<li><b>bar</b></li>\n</ol>')
 
     def test_subtag_no_newlines(self):
         'prevent generation of newlines against default'
@@ -429,7 +456,7 @@ class TestCase(unittest.TestCase):
             unicode = str
             TEST = 'euro €'
         h.p(TEST)
-        self.assertEquals(unicode(h), '<p>%s</p>'%TEST)
+        self.assertEquals(unicode(h), '<p>%s</p>' % TEST)
 
     def test_table(self):
         'multiple "with" context blocks'
@@ -449,17 +476,17 @@ if __name__ == '__main__':
 
 
 # Copyright (c) 2009 eKit.com Inc (http://www.ekit.com/)
-# 
+#
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
 # in the Software without restriction, including without limitation the rights
 # to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 # copies of the Software, and to permit persons to whom the Software is
 # furnished to do so, subject to the following conditions:
-# 
+#
 #  The above copyright notice and this permission notice shall be included in
 #  all copies or substantial portions of the Software.
-# 
+#
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 # IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 # FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
